/**
 * SwingTiffy.java                                                    <br>
 * Copyright (c) 1998-2005 Helge Hackbarth, All Rights Reserved.     <br>
 * Main class for an applet demonstrating the Tiffy Java Toolkit      <br>
 * using Swing GUI. The program can be used as applet and as          <br>
 * standalone application. Refer to getParameterInfo() to learn about <br>
 * commandline / parameter options. If running standalone, the        <br>
 * parameter-names can be suppressed (e.g. you may start              <br>
 *      java SwingTiffy image.tif -f                                  <br>
 * instead of                                                         <br>
 *      java SwingTiffy Filename=image.tif Options=-f                 <br>
 * To get information about supported options please refer to         <br>
 * ImagePanel.parseOptions().                                         <br>
 * THE AUTHOR MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
 * OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. THE AUTHOR SHALL NOT BE LIABLE FOR ANY
 * DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * @author Helge Hackbarth
 * @version 2.5.6
 */
import java.awt.*;
import java.awt.event.*;
import java.net.URL;
// Depending on the version of Swing classes you have to use
// either the import statement for com.sun.java.swing.* (e.g. in Jbuilder2)
// or for javax.swing.*; (e.g. for JDK 1.2)
// import com.sun.java.swing.*; // packages for Swing 1.0 classes (remove if not used)
import javax.swing.*;        // packages for Swing 1.1 classes (remove if not used)
import ach.ImagePanel;
import ach.JTiffyToolBar;
import ach.file.ParamUtil;

//import com.sun.java.swing.UIManager;
public class SwingTiffy extends JApplet implements WindowListener {
  boolean isStandalone = false;
  final String PARAM_Filename = "Filename";
  final String PARAM_Options = "Options";
  String m_Filename = "";
  String m_Options = "";
  ImagePanel imagePanel;
  JTiffyToolBar toolBar;
  GridBagLayout gb = new GridBagLayout();
  GridBagConstraints gc = new GridBagConstraints();

  // STANDALONE APPLICATION SUPPORT
  //   The GetParameter() method is a replacement for the getParameter() method
  // defined by Applet. This method returns the value of the specified parameter;
  // unlike the original getParameter() method, this method works when the applet
  // is run as a standalone application, as well as when run within an HTML page.
  // This method is called by GetParameters().
  //---------------------------------------------------------------------------


  String GetParameter(String strName, String args[])
  {
    if (args == null)
    {
      // Running within an HTML page, so call original getParameter().
      //-------------------------------------------------------------------
      return getParameter(strName);
    }

    // Running as standalone application, so parameter values are obtained from
    // the command line. The user specifies them as follows:
    //
    //  JView Tiffy param1=<val> param2=<"val with spaces"> ...
    //-----------------------------------------------------------------------
    int i;
    String strArg  = strName + "=";
    int strArgLen   = strArg.length();
    String strValue = null;

    for (i = 0; i < args.length; i++)
    {
      if ((args[i].length() >= strArgLen) && (strArg.equalsIgnoreCase(args[i].substring(0, strArgLen))))
      {
        // Found matching parameter on command line, so extract its value.
        // If in double quotes, remove the quotes.
        //---------------------------------------------------------------
        strValue= args[i].substring(strArg.length());
        if (strValue.startsWith("\""))
        {
          strValue = strValue.substring(1);
          if (strValue.endsWith("\""))
            strValue = strValue.substring(0, strValue.length() - 1);
        }
      }
    }

    return strValue;
  }

  // STANDALONE APPLICATION SUPPORT
  //   The GetParameters() method retrieves the values of each of the applet's
  // parameters and stores them in variables. This method works both when the
  // applet is run as a standalone application and when it's run within an HTML
  // page.  When the applet is run as a standalone application, this method is
  // called by the main() method, which passes it the command-line arguments.
  // When the applet is run within an HTML page, this method is called by the
  // init() method with args == null.
  //---------------------------------------------------------------------------
  void GetParameters(String args[])
  {
    // Query values of all Parameters
    //--------------------------------------------------------------
    String param;

    // Filename: Name of image file
    //--------------------------------------------------------------
    param = GetParameter(PARAM_Filename, args);
    if (param != null)
      m_Filename = param;

    // Options: Options for display
    //--------------------------------------------------------------
    param = GetParameter(PARAM_Options, args);
    if (param != null)
      m_Options = param;


    if (args != null) {
      // search for unnamed parameters
      // if it starts without '-' or '/' it is interpreted as filename
      // otherwise it will be added to the options string
      for (int i = 0; i < args.length; i++) {
        if (args[i].indexOf('=') < 0) {
          // Found unnamed parameter on command line
          // If in double quotes, remove the quotes.
          //----------------------------------------
          param = args[i];
          if (param.startsWith("\"")) {
            param = param.substring(1);
            if (param.endsWith("\""))
              param = param.substring(0, param.length() - 1);
          }
          if (param.startsWith("-") || param.startsWith("/"))
            if (m_Options.length() == 0)
              m_Options = m_Options + param;
            else
              m_Options = m_Options + " " + param;
          else
            if (m_Filename == "")
              m_Filename = param;
        }
      }
    }
  }

  // applet constructor
  public SwingTiffy() {
  }

  // initialise applet
  public void init() {
    if (!isStandalone)
      GetParameters(null);
    // To allow loading toolkit related ressources compatible to Java 1.0
    // use ParamUtil.retrieveCodeBase(this) for applets and
    // ParamUtil.retrieveProgramPath(this, defaultPath) for standalone applications
    // This is important for applets running inside Netscape Navigator as well
    ParamUtil.retrieveCodeBase(this);
    toolBar = new JTiffyToolBar();
    try { jbInit(); }
    catch (Exception e) { e.printStackTrace(); }
    try {
      if (isStandalone)
        if ((m_Filename != null) && (m_Filename.toLowerCase().startsWith("http:")))
          imagePanel = new ImagePanel(this, new URL(m_Filename), m_Options, false);
        else
          imagePanel = new ImagePanel(this, m_Filename, m_Options, false);
      else
        imagePanel = new ach.ImagePanel(this, new URL(getCodeBase(), m_Filename), m_Options, false);
      if (!isStandalone) // reserve insets for 3D-border
        gc.insets = new Insets(0,2,2,2);
      gc.weighty = 1;
      gb.setConstraints(imagePanel,gc);
      this.getContentPane().add(imagePanel);
      toolBar.setImagePanel(imagePanel); // allow toolbar to process events for imagePanel
      // Disable LightWeightPopup for Swing 1.1 tooltips
      // otherwise the tooltip might appear invisible (behind imagePanel)
      ToolTipManager.sharedInstance().setLightWeightPopupEnabled(false);
    } catch (java.net.MalformedURLException mue)
    { mue.printStackTrace(); }
  }

  //static {
  //  try {
  //    //UIManager.setLookAndFeel(new com.sun.java.swing.plaf.metal.MetalLookAndFeel());
  //    //UIManager.setLookAndFeel(new com.sun.java.swing.plaf.motif.MotifLookAndFeel());
  //    UIManager.setLookAndFeel(new com.sun.java.swing.plaf.windows.WindowsLookAndFeel());
  //  }
  //  catch (Exception e) {}
  //}

  // initialise components
  private void jbInit() throws Exception {
    this.setSize(728,420);
    // set GridBagLayout with 2 pixel insets in order to paint a 3D border
    this.getContentPane().setLayout(gb);
    if (!isStandalone) // insets for 3D-border
      gc.insets = new Insets(2,2,0,2);
    gc.fill = GridBagConstraints.BOTH;
    gc.weightx = 1;
    gc.gridwidth = GridBagConstraints.REMAINDER;
    gb.setConstraints(toolBar,gc);
    this.getContentPane().add(toolBar);
  }

  //called when applet is started
  public void start() {
    if ((m_Filename == null) || (m_Filename == "")) {
      Frame dummyFrame = new Frame("");
      ach.OkBox ok = new ach.OkBox(dummyFrame,
      "Missing filename","Please specify image name(s) as parameters.");
      ok.show();
      destroy();
      try { System.exit(0); } catch (Exception e) { }
    }
  }

  //called when applet is stopped
  public void stop() {
    // As a modal dialog window opened from a browser applet does not
    // behave as expected, dispose it, if it is still open
    if (ach.OkBox.currentOkBox != null) ach.OkBox.currentOkBox.cleanUp();
    System.gc(); // force garbage collection
  }

  //called when applet is destroyed
  public void destroy() {
    if (imagePanel != null)
      imagePanel.cleanUp();
    if(System.getProperty("java.version").equals("1.1.5")) {
      // Dan Kok's workaround for Netscape 4.5-4.7/MSWINDOWS freezing bug
      try { Thread.sleep(2000); } catch(InterruptedException ex) { }
    }
  }

  //get applet information
  public String getAppletInfo() {
    return "Applet-Information";
  }

  //get parameter-infos
  public String[][] getParameterInfo() {
    String pinfo[][] =
    {
      {"FileName", "String", "Image file name"},
      {"Options", "String", "Optional parameters"},
    };
    return pinfo;
  }

  // main-method (called if run as standalone application)
  public static void main(String[] args) {
    SwingTiffy applet = new SwingTiffy();
    applet.isStandalone = true;
    JFrame frame = new JFrame();
    frame.addWindowListener(applet);
    frame.getContentPane().add(applet, BorderLayout.CENTER);
    applet.GetParameters(args);
    frame.setTitle("Tiffy Demo using Swing UI - " + applet.m_Filename);
    applet.init();
    applet.start();
    frame.setSize(728,420);
    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
    // frame.setLocation((d.width - frame.getSize().width) / 2, (d.height - frame.getSize().height) / 2);
    frame.setVisible(true);
  }

  // paint handler
  public void paint(Graphics g)
  {
    super.paint(g);
    if ((imagePanel != null) && (!imagePanel.frameUpdated))
      imagePanel.repaint(); // otherwise some repaints don't get through to the imagepanel
    if (!isStandalone) { // draw 3D-border around applet
      int w = size().width;
      int h = size().height;
      g.setColor(Color.lightGray);
      g.drawLine(0,0,w-2,0); g.drawLine(0,0,0,h-2);
      g.setColor(Color.white);
      g.drawLine(1,1,w-3,1); g.drawLine(1,1,1,h-3);
      g.setColor(Color.gray);
      g.drawLine(1,h-2,w-2,h-2); g.drawLine(w-2,1,w-2,h-2);
      g.setColor(Color.black);
      g.drawLine(0,h-1,w-1,h-1); g.drawLine(w-1,0,w-1,h-1);
    }
    if (imagePanel != null)
      imagePanel.frameUpdated = false;
  }

  // implement all methods of the WindowListener interface
  public void windowClosed(WindowEvent event) {    }
  public void windowDeiconified(WindowEvent event) {    }
  public void windowIconified(WindowEvent event) {    }
  public void windowActivated(WindowEvent event) {    }
  public void windowDeactivated(WindowEvent event) {    }
  public void windowOpened(WindowEvent event) {    }
  public void windowClosing(WindowEvent event) {
    destroy();
    System.exit(0);
  }

}


